/*

  G. Rinaldi -- Calcolo delle soglie di anomalia secondo l'art. 97 del
                Codice dei Contratti Pubblici (Revisione 06/2020)
  
		Versione: 14 luglio 2020

  Istruzioni:

  CalcoloAnomalie -r  < [file con l'elenco delle % di ribasso]

  oppure

  CalcoloAnomalie -v  < [file con la base d'asta seguita dall'elenco dei 
                         valori offerti]

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>

int compare_function (const void *a, const void *b) {
  int *x = (int *) a;
  int *y = (int *) b;
  return *x - *y;
}

int main (int argc, char **argv )
{
  int v[100], n, nn, ns, i0, i1, d, j, valori;
  double r, base, ribasso[100], scarto, anomalia, somma_ribassi, media;

  if (argc != 2 || (strncmp(argv[1],"-r",2) && strncmp(argv[1],"-v",2))) {
    printf ("Usage: %s -r | -v < file\n", argv[0]);
    exit (1);
  }

  if (strncmp(argv[1],"-r",2)) valori = 1;
  else valori = 0;
  
  n = valori ? -1 : 0;
  while (!feof(stdin)) {
    scanf ("%lf\n", &r);
    if (r < 0.0) {
      printf ("Valore negativo (%lf) non ammesso: viene trascurato.\n", r);
      continue;
    }
    if (n < 0) {
      base = (int) (r * 100 + 0.5);
      if ((int) base < 1) {
	printf ("La base d'asta non èuò essere nulla.\n");
	exit (1);
      }
      printf ("Base d'asta (in centesimi di euro): %d\n", (int) base);
    }
    else {
      if (valori) {
	if (r > base / 100.0) {
	  printf ("Valore (%lf) superiore alla base d'asta: "
		  "viene trascurato.\n", r);
	  continue;
	} else v[n] = (int) (r * 100 + 0.5); 
      } else {
	if (r > 100.0) {
	  printf ("Ribasso (%lf) superiore al 100%%: "
		  "viene trascurato.\n", r);
	  continue;
	} else v[n] = (int) (r * 1000); 
      }
    }
    n++; 
  }

  if (n < 10) {
    printf ("\nArt. 97, comma 8, ultimo periodo: numero offerte inferiore "
	    "a 10\n");
    exit (1);
  }
  
  qsort (v, n, sizeof(int), compare_function);

  printf ("\nEliminazione delle code (art. 97 comma 2.a e 2-bis.a):\n");
  d = ceil (n / 10.0);
  i0 = d;
  while (v[i0] == v[i0 - 1]) i0++;
  i1 = n - d;
  while (v[i1] == v[i1 - 1]) i1--;

  printf ("Coda inferiore:\n");
  for (j = 0; j < i0; j++) printf ("%2d) %6d\n", j + 1, v[j]);
  printf ("\nCorpo centrale:\n");
  for (; j < i1; j++) printf ("%2d) %6d\n", j + 1, v[j]);
  printf ("\nCoda superiore:\n");
  for (; j < n; j++) printf ("%2d) %6d\n", j + 1, v[j]);
  printf ("\n");
  
  if (i0 >= i1) {
    printf ("Dopo l'eliminazione delle code non sopravvive alcun valore: "
	    "il calcolo non può proseguire.\n");
    exit (1);
  }
  
  for (j = i0, nn = 0; j < i1; j++, nn++) {
    if (valori)
      ribasso[nn] = floor(100000.0 - (v[j] / base) * 100000);
    else
      ribasso[nn] = v[j];
    ribasso[nn] /=  1000.0;
  }
    
  printf ("\nRibassi considerati:\n");
  for (j = 0; j < nn; j++) printf ("%2d) %7.3lf\n", j + 1, ribasso[j]);
  printf ("\n");

  // Calcolo media ribassi (commi 2.a e 2-bis.a)
  somma_ribassi = 0.0;
  for (j = 0; j < nn; j++) somma_ribassi += ribasso[j];
  
  media = somma_ribassi / nn;

  printf ("Media ribassi = %lf\n", media);

  // Calcolo scarto medio ribassi che superano la media

  scarto = 0.0;
  ns = 0;
  for (j = 0; j < nn; j++)
  if (ribasso[j] > media) {
    scarto += (ribasso[j] - media);
    ns++;
  }
  if (ns) scarto /= ns;
  printf ("Media scarti dei ribassi superiori alla media = %lf\n", scarto);

  if (n < 15) { // Art. 96 comma 2-bis
    printf ("Meno di 15 offerte => Art. 96 comma 2-bis\n");
    double rapporto;

    if (media > 0) rapporto = scarto / media;
    else rapporto = 1.0;

    if (rapporto <= 0.15) anomalia = media * 1.2;
    else anomalia = media + scarto;
  } else { // Art. 96 comma 2
    printf ("Almeno 15 offerte => Art. 96 comma 2\n");

    printf ("Somma ribassi = %lf\n", somma_ribassi);
    
    int a = (int) (somma_ribassi * 100);
    int b = (int) (a % 100) / 10;
    int c = a % 10;

    printf ("Prime due cifre dopo la virgola = %d  %d\n", b, c);
    
    double perc = b * c / 100.00;
    anomalia = media + scarto * (1.0 - perc);

    printf ("La media scarti che superano la media è decrementata del "
	    "fattore = %lf%%\n", perc);

  }
    if (valori)
      printf ("\nSoglia di anomalia (valore economico) = %lf\n\n",
	      (100.0 - anomalia) * base / 10000.0);
    else 
      printf ("\nSoglia di anomalia (%% di ribasso) = %lf\n\n", anomalia);

    exit (0);
}
